@@ -11,22 +11,46 @@ import numpy as np
 import pandas as pd
 import h5py
 import shutil
+import os
 
 from IPython.display import display
 from datetime import datetime
 
+from ..version import get_git_version
+from importlib.metadata import version, PackageNotFoundError
 
-class Dataset():
-    """THE DATASETS CLASS IS MEANT TO BE READ VIA THE gwaihir.gui CLASS !!"""
+try:
+    bcdi_version = version("bcdi")
+except PackageNotFoundError:
+    bcdi_version = None
+
+try:
+    pynx_version = version("pynx")
+except PackageNotFoundError:
+    pynx_version = None
+
+try:
+    gwaihir_version = version("gwaihir")
+except PackageNotFoundError:
+    gwaihir_version = None
+
+
+class Dataset:
+    """
+    Created for Gwaihir
+    Allows to save the dataset at a CXI file.
+    """
 
     def __init__(self, scan, sample_name, data_dir, root_folder):
-        """Initialiaze the Dataset class, some metadata can be associated as
+        """
+        Initialiaze the Dataset class, some metadata can be associated as
         well.
         """
         self.scan = scan
         self.sample_name = sample_name
         self.data_dir = data_dir
         self.root_folder = root_folder
+        self._gwaihir_version = get_git_version()
 
     def __repr__(self):
         return "Dataset {}{}.\n".format(
@@ -37,20 +61,36 @@ class Dataset():
     def __str__(self):
         return repr(self)
 
-    def to_cxi(self, cxi_filename, reconstruction_filename=False):
+    def to_cxi(
+        self,
+        cxi_filename,
+        reconstruction_filename=None,
+        strain_output_file=None,
+    ):
         """
         Save all the parameters used in the data analysis with a specific
         architecture based on NeXuS.
+
+        :param cxi_filename: .cxi file that contains the preprocessed data,
+         created thanks to PyNX.
+         This file is used as base for the final cxi file.
+        :param reconstruction_filename: .cxi or .h5 file, output of phase retrieval
+         chosen for postprocessing.
+        :param strain_output_file: .h5 file, output from postprocessing
         """
+
+        # Create final file name
         final_data_path = f"{self.scan_folder}{self.sample_name}{self.scan}.cxi"
-        shutil.copy(cxi_filename,
-                    final_data_path,
+
+        # Copy cxi file, and use it as starter for the end file
+        shutil.copy(cxi_filename,  # src
+                    final_data_path,  # dest
                     )
 
-        if isinstance(reconstruction_filename, str):
+        # Add info from postprocessing if possible
+        if os.path.isfile(reconstruction_filename):
             with h5py.File(reconstruction_filename, "r") as reconstruction_file, \
                     h5py.File(final_data_path, "a") as final_file:
-                # Real space data is already here from PyNX
 
                 print("\nSaving phase retrieval output ...")
 
@@ -65,6 +105,12 @@ class Dataset():
                         '/entry_1/image_1/', final_file["entry_1"],
                         name="image_2")
 
+                # Save file name
+                final_file["entry_1"]["image_2"].create_dataset(
+                    "reconstruction_filename",
+                    data=reconstruction_filename
+                )
+
                 # Update params if reconstruction file results
                 # from mode decomposition
                 if reconstruction_filename.endswith(".h5"):
@@ -149,7 +195,11 @@ class Dataset():
 
         # Add GUI data
         with h5py.File(final_data_path, "a") as f:
-            # Parameters
+            # Save Gwaihir version
+            f.create_dataset("gwaihir_version", data="Gwaihir %s" %
+                             self._gwaihir_version)
+
+            # Create parameter groups
             try:
                 data_3 = f.create_group("entry_1/data_3/")
                 f["entry_1"]["data_3"].attrs['NX_class'] = 'NXdata'
@@ -374,8 +424,8 @@ class Dataset():
                 linearized_transformation_matrix.create_dataset(
                     "sample_offsets", data=self.sample_offsets)
                 linearized_transformation_matrix.create_dataset(
-                    "sdd", data=self.sdd)
-                linearized_transformation_matrix["sdd"].attrs['units'] = 'm'
+                    "detector_distance", data=self.detector_distance)
+                linearized_transformation_matrix["detector_distance"].attrs['units'] = 'm'
                 linearized_transformation_matrix.create_dataset(
                     "energy", data=self.energy)
                 linearized_transformation_matrix["energy"].attrs['units'] = 'keV'
@@ -577,43 +627,44 @@ class Dataset():
 
             # Save strain output
             try:
-                image_3.create_dataset("strain_analysis_output_file",
-                                       data=self.strain_output_file)
+                if os.path.isfile(strain_output_file):
+                    image_3.create_dataset("strain_analysis_output_file",
+                                           data=strain_output_file)
 
-                with h5py.File(self.strain_output_file, "r") as fi:
-                    image_3.create_dataset("amplitude",
-                                           data=fi["output"]["amp"][:],
-                                           chunks=True,
-                                           shuffle=True,
-                                           compression="gzip")
+                    with h5py.File(strain_output_file, "r") as fi:
+                        image_3.create_dataset("amplitude",
+                                               data=fi["output"]["amp"][:],
+                                               chunks=True,
+                                               shuffle=True,
+                                               compression="gzip")
 
-                    image_3.create_dataset("phase",
-                                           data=fi["output"]["phase"][:],
-                                           chunks=True,
-                                           shuffle=True,
-                                           compression="gzip")
+                        image_3.create_dataset("phase",
+                                               data=fi["output"]["phase"][:],
+                                               chunks=True,
+                                               shuffle=True,
+                                               compression="gzip")
 
-                    image_3.create_dataset("bulk",
-                                           data=fi["output"]["bulk"][:],
-                                           chunks=True,
-                                           shuffle=True,
-                                           compression="gzip")
+                        image_3.create_dataset("bulk",
+                                               data=fi["output"]["bulk"][:],
+                                               chunks=True,
+                                               shuffle=True,
+                                               compression="gzip")
 
-                    image_3.create_dataset("strain",
-                                           data=fi["output"]["strain"][:],
-                                           chunks=True,
-                                           shuffle=True,
-                                           compression="gzip")
+                        image_3.create_dataset("strain",
+                                               data=fi["output"]["strain"][:],
+                                               chunks=True,
+                                               shuffle=True,
+                                               compression="gzip")
 
-                    image_3.create_dataset("voxel_sizes",
-                                           data=fi["output"]["voxel_sizes"])
+                        image_3.create_dataset("voxel_sizes",
+                                               data=fi["output"]["voxel_sizes"])
 
-                    image_3.create_dataset("q_com",
-                                           data=fi["output"]["q_com"])
+                        image_3.create_dataset("q_com",
+                                               data=fi["output"]["q_com"])
 
-                image_3.attrs['signal'] = 'phase'
+                    image_3.attrs['signal'] = 'phase'
 
-            except AttributeError:
+            except (AttributeError, TypeError):
                 print("Could not save strain output")
 
             # Create data_3 link
