@@ -1,13 +1,16 @@
-from tornado.ioloop import PeriodicCallback
-from skimage.measure import marching_cubes
-from scipy.spatial.transform import Rotation
-from scipy.interpolate import RegularGridInterpolator
+import warnings
 import numpy as np
 import os
 import h5py as h5
 import tables as tb
 import glob
 
+from tornado.ioloop import PeriodicCallback
+
+from skimage.measure import marching_cubes
+from scipy.spatial.transform import Rotation
+from scipy.interpolate import RegularGridInterpolator
+
 import matplotlib.pyplot as plt
 from matplotlib import cm
 from matplotlib.colors import LogNorm, Normalize
@@ -17,7 +20,6 @@ import ipywidgets as widgets
 from ipywidgets import interact, Button, Layout, interactive, fixed
 from IPython.display import Markdown, Latex, clear_output
 from IPython.core.display import display, HTML
-# import ipyfilechooser
 import ipyvolume as ipv
 
 from bokeh.plotting import figure, show, output_file
@@ -25,20 +27,18 @@ from bokeh.layouts import row, column
 from bokeh.io import output_notebook
 from bokeh.models import ColumnDataSource, ColorBar, LogColorMapper, LinearColorMapper
 from bokeh.models import BoxEditTool, HoverTool, CrosshairTool, LassoSelectTool
-from bokeh.models import CustomJS, Slider, SaveTool
+from bokeh.models import CustomJS, Slider, SaveTool, Select, RadioButtonGroup
 from bokeh.models.widgets import Tabs, Panel
 import bokeh.palettes as bp
+import panel as pn
+pn.extension()
 
-import warnings
 warnings.filterwarnings("ignore")
 
-output_notebook()
 
-# Classes
-
-
-class Plotter():
-    """Class based on interactive functions for plotting.
+class Plotter:
+    """
+    Class based on interactive functions for plotting.
     Gets data array from file and plot if specified
     """
 
@@ -46,7 +46,7 @@ class Plotter():
         self,
         filename=None,
         data_array=None,
-        plot=False,
+        plot="slices",
         log=False,
         cmap="YlGnBu_r",
         figsize=(10, 10),
@@ -273,7 +273,8 @@ class Plotter():
 
 
 class ThreeDViewer(widgets.Box):
-    """Widget to display 3D objects from CDI optimisation, loaded from a result
+    """
+    Widget to display 3D objects from CDI optimisation, loaded from a result
     CXI file or a mode file.
 
     Quickly adapted from @Vincent Favre Nicolin (ESRF)
@@ -807,7 +808,10 @@ def plot_data(
     cmap="YlGnBu_r",
     title=None,
 ):
-    """Create figure based on the data dimensions.
+    """
+    Create figure based on the data dimensions.
+    Data in 2D is here always plotted as an image
+    Data in 3D is plotted as 3 slices or directly in 3D
 
     :param data_array: np.ndarray to plot
     :param figsize: default (10, 10)
@@ -919,216 +923,255 @@ def plot_data(
                 plt.show()
 
     elif data_dimensions == 3:
-        @interact(
-            axplot=widgets.Dropdown(
-                options=[
-                    ("z", "xy"),
-                    ("x", "yz"),
-                    ("y", "xz")
-                ],
-                value="xy",
-                description='Slice along:',
-                style={'description_width': 'initial'}),
-            ComplexNumber=widgets.ToggleButtons(
-                options=["Real", "Imaginary", "Module", "Phase"],
-                value="Module",
-                description='Plotting options:',
-                tooltip=['Plot only contour or not', "", ""],
-                style={'description_width': 'initial'})
-        )
-        def plot_3d(
-            axplot,
-            ComplexNumber
+
+        def get_data_slice(
+            data,
+            axis="x",
+            index=0,
+            data_type="Module",
+            scale="linear",
         ):
+            # Project on specific index
+            if axis == "x":
+                dt = data[index, :, :]
+            elif axis == "y":
+                dt = data[:, index, :]
+            elif axis == "z":
+                dt = data[:, :, index]
 
-            # Decide what we want to plot
-            if ComplexNumber == "Real":
-                data = np.real(data_array)
-            elif ComplexNumber == "Imaginary":
-                data = np.imag(data_array)
-            elif ComplexNumber == "Module":
-                data = np.abs(data_array)
-            elif ComplexNumber == "Phase":
-                data = np.angle(data_array)
+            # Data type
+            if data_type == "Real":
+                dt = np.real(dt)
+            elif data_type == "Imaginary":
+                dt = np.imag(dt)
+            elif data_type == "Module":
+                dt = np.abs(dt)
+            elif data_type == "Phase":
+                dt = np.angle(dt)
 
-            # Take the shape of that array along 2 axis
-            if axplot == "xy":
-                r = np.shape(data[0, 0, :])
+            # Scale
+            if scale == "logarithmic":
+                dt = np.where(dt > 0, np.log(dt), 0)
 
-            elif axplot == "yz":
-                r = np.shape(data[:, 0, 0])
+            lin_color_mapper.high = np.max(dt)
 
-            elif axplot == "xz":
-                r = np.shape(data[0, :, 0])
+            return dt
 
-            @interact(
-                i=widgets.IntSlider(
-                    min=0,
-                    max=r[0]-1,
-                    step=1,
-                    description=f'Index [0; {r[0]-1}]:',
-                    orientation='horizontal',
-                    continuous_update=False,
-                    readout=True,
-                    readout_format='d',
-                    layout=Layout(width='80%'),
-                    style={'description_width': 'initial'},
-                ),
-                # PlottingOptions=widgets.ToggleButtons(
-                #     options=[("2D image", "2D"),
-                #              ("2D image with contour", "2DC"),
-                #              # ("3D surface plot", "3D")
-                #              ],
-                #     value="2D",
-                #     description='Plotting options',
-                #     disabled=False,
-                #     button_style='',
-                #     tooltip=['Plot only contour or not', "", ""],
-                # ),
-                # scale=widgets.ToggleButtons(
-                #     options=["linear", "logarithmic"],
-                #     value="linear",
-                #     description='Scale',
-                #     disabled=False,
-                #     style={'description_width': 'initial'}),
+        output_notebook()
+
+        # Get bokeh palette from colormap
+        TOOLTIPS = [
+            ("x", "$x"),
+            ("y", "$y"),
+            ("value", "@data"),
+        ]
+
+        # List of compatible cmaps in bokeh
+        bokey_cmaps = [
+            p for p in bp.__palettes__ if p.endswith("256")
+        ]
+
+        palette = "Magma256"
+        for p in bokey_cmaps:
+            if cmap[1:] in p:  # skip capital letter
+                palette = p
+                print("Changing cmap to", palette)
+
+        # Figure
+        fig = figure(
+            x_axis_label="x",
+            y_axis_label="y",
+            toolbar_location="above",
+            toolbar_sticky=False,
+            tools="pan, wheel_zoom, box_zoom, reset, undo, redo, crosshair, hover, save",
+            active_scroll="wheel_zoom",
+            active_tap="auto",
+            active_drag="box_zoom",
+            active_inspect="auto",
+            tooltips=TOOLTIPS
+        )
+
+        # Define source
+        source = ColumnDataSource(
+            data=dict(
+                data=[np.abs(data_array[0, :, :])],
+                dw=[data_array.shape[1]],
+                dh=[data_array.shape[2]],
+                index=[0],
+                axis=["x"],
+                data_type=["Module"],
+                scale=["linear"],
             )
-            def PickLastAxis(i,
-                             # PlottingOptions,
-                             # scale
-                             ):
-                if axplot == "xy":
-                    dt = data[:, :, i]
-                    x_label = "y"
-                    y_label = "x"
-                elif axplot == "yz":
-                    dt = data[i, :, :]
-                    x_label = "z"
-                    y_label = "y"
-                elif axplot == "xz":
-                    dt = data[:, i, :]
-                    x_label = "z"
-                    y_label = "x"
+        )
 
-                else:
-                    raise TypeError("Choose xy, yz or xz as axplot.")
-                ## No BOKEH ##
-                # # Create figure
-                # plt.close()
-                # print("Figure size defaulted to", figsize)
-                # fig, ax = plt.subplots(1, 1, figsize=figsize)
+        # Index
+        def callback_change_index(attr, old, new):
+            # Compute data
+            dt = get_data_slice(
+                data=data_array,
+                axis=source.data["axis"][0],
+                index=new,
+                data_type=source.data["data_type"][0],
+                scale=source.data["scale"][0],
+            )
 
-                # # Get scale
-                # log = scale == "logarithmic"
+            # Save new values
+            source.data["data"] = [dt]
+            source.data["index"] = [new]
 
-                # # Plot 2D image in interactive environment
-                # img = plot_2d_image(two_d_array=dt, log=log, fontsize=fontsize,
-                #                     fig=fig, ax=ax, cmap=cmap, title=title,
-                #                     x_label=x_label, y_label=y_label)
+        slider_index = Slider(
+            start=0,
+            end=data_array.shape[0]-1,
+            value=0,
+            step=1,
+            title="Position",
+            orientation = "vertical",
+        )
+        slider_index.on_change('value', callback_change_index)
 
-                # # Create axis for colorbar
-                # cbar_ax = make_axes_locatable(ax).append_axes(
-                #     position='right', size='5%', pad=0.1)
+        # Axis
+        def callback_change_axis(attr, old, new):
+            # Get axis for projection
+            new_axis = ["x", "y", "z"][new]
 
-                # # Create colorbar
-                # cbar = fig.colorbar(mappable=img, cax=cbar_ax)
+            # Compute data
+            dt = get_data_slice(
+                data=data_array,
+                axis=new_axis,
+                index=source.data["index"][0],
+                data_type=source.data["data_type"][0],
+                scale=source.data["scale"][0],
+            )
 
-                # # Show figure
-                # plt.tight_layout()
-                # plt.show()
-                # plt.close()
+            # Save new values
+            source.data["data"] = [dt]
+            source.data["axis"] = [new_axis]
+            source.data["dw"][0] = dt.shape[0]
+            source.data["dh"][0] = dt.shape[1]
 
-                ## BOKEH ##
-                TOOLTIPS = [
-                    ("x", "$x"),
-                    ("y", "$y"),
-                    ("value", "@image"),
-                ]
+            # Get new axis names and slider range
+            if new_axis == "x":
+                slider_range = data.shape[0]
+                fig.axis[0].axis_label = "y"
+                fig.axis[1].axis_label = "z"
 
-                # List of compatible cmaps in bokeh
-                palette = "Viridis256"
-                bokey_cmaps = [
-                    p for p in bp.__palettes__ if p.endswith("256")
-                ]
-                for p in bokey_cmaps:
-                    if cmap[1:] in p:  # skip capital letter
-                        palette = p
-                        print("Changing cmap to", p)
+            elif new_axis == "y":
+                slider_range = data.shape[1]
+                fig.axis[0].axis_label = "x"
+                fig.axis[1].axis_label = "z"
 
-                panels = []
+            elif new_axis == "z":
+                slider_range = data.shape[2]
+                fig.axis[0].axis_label = "x"
+                fig.axis[1].axis_label = "y"
 
-                for axis_type, cmapper in zip(
-                    ["Linear scale", "Logarithmic scale"],
-                    [LinearColorMapper, LogColorMapper]
-                ):
-                    # Figure
-                    fig = figure(
-                        title=f"Data slice on ({x_label}, {y_label}) for i={i}",
-                        x_axis_label=x_label,
-                        y_axis_label=y_label,
-                        toolbar_location="above",
-                        toolbar_sticky=False,
-                        tools="pan, wheel_zoom, box_zoom, reset, undo, redo, crosshair, hover, save",
-                        active_scroll="wheel_zoom",
-                        active_tap="auto",
-                        active_drag="box_zoom",
-                        active_inspect="auto",
-                        tooltips=TOOLTIPS,
-                        match_aspect=True,
-                    )
+            # Change slider range
+            slider_index.end = slider_range-1
 
-                    # Color bar
-                    if axis_type == "Linear scale":
-                        low = np.min(dt)
-                    else:
-                        low = 0.1 if np.min(dt) == 0 else np.min(dt)
+        select_axis = RadioButtonGroup(
+            labels=["x", "y", "z"],
+            active=0,
+        )
+        select_axis.on_change('active', callback_change_axis)
 
-                    color_mapper = cmapper(
-                        palette=palette,
-                        low=low,
-                        high=np.max(dt),
-                    )
-                    color_bar = ColorBar(color_mapper=color_mapper)
-                    fig.add_layout(color_bar, 'right')
+        # Data type
+        def callback_change_data_type(attr, old, new):
+            # Get data type
+            new_data_type = ["Real", "Imaginary", "Module", "Phase"][new]
 
-                    # Image
-                    image = fig.image(
-                        image=[dt],
-                        x=0,
-                        y=0,
-                        dw=dt.shape[0],
-                        dh=dt.shape[1],
-                        color_mapper=color_mapper,
-                    )
+            # Compute data
+            dt = get_data_slice(
+                data=data_array,
+                axis=source.data["axis"][0],
+                index=source.data["index"][0],
+                data_type=new_data_type,
+                scale=source.data["scale"][0],
+            )
 
-                    # Background
-                    fig.background_fill_color = "white"
-                    fig.background_fill_alpha = 0.5
+            # Save new values
+            source.data["data"] = [dt]
+            source.data["data_type"] = [new_data_type]
 
-                    # Title
-                    # fig.title.text_color = "olive"
-                    fig.title.text_font = "futura"
-                    fig.title.text_font_style = "bold"
-                    fig.title.text_font_size = "15px"
+        select_data_type = RadioButtonGroup(
+            labels=["Real", "Imaginary", "Module", "Phase"],
+            active=2,
+        )
+        select_data_type.on_change('active', callback_change_data_type)
 
-                    panel = Panel(child=fig, title=axis_type)
-                    panels.append(panel)
+        # Color bar
+        def callback_change_cbar(attr, old, new):
+            # Get new cbar
+            new_cbar = ["linear", "logarithmic"][new]
 
-                tabs = Tabs(tabs=panels)
+            # Compute data
+            dt = get_data_slice(
+                data=data_array,
+                axis=source.data["axis"][0],
+                index=source.data["index"][0],
+                data_type=source.data["data_type"],
+                scale=new_cbar,
+            )
 
-                show(tabs)
+            # Save new values
+            source.data["data"] = [dt]
+            source.data["scale"] = [new_cbar]
 
-                ## CONTOUR ##
+        select_cbar = RadioButtonGroup(
+            labels=["linear", "logarithmic"],
+            active=0,
+        )
+        select_cbar.on_change('active', callback_change_cbar)
 
-                # if PlottingOptions == "2D":
-                # elif PlottingOptions == "2DC":
-                #     # Show contour plot instead
+        # Background
+        fig.background_fill_color = "white"
+        fig.background_fill_alpha = 0.5
 
-                #     plt.close()
+        # Title
+        fig.title.text_font = "futura"
+        fig.title.text_font_style = "bold"
+        fig.title.text_font_size = "15px"
 
-                #     log = True if scale == "logarithmic"  else False
-                #     img = plot_2d_image_contour(two_d_array=dt, log=log)
+        # Color bars
+        lin_color_mapper = LinearColorMapper(
+            palette=palette,
+            low=0,
+            high=np.max(source.data["data"][0]),
+        )
+        lin_color_bar = ColorBar(color_mapper=lin_color_mapper)
 
-                #     plt.show()
+        fig.add_layout(lin_color_bar, 'right')
+
+        # Image
+        image = fig.image(
+            image="data",
+            source=source,
+            x=0,
+            y=0,
+            dw="dw",
+            dh="dh",
+            color_mapper=lin_color_mapper,
+        )
+
+        # Create app layout
+        app = pn.Column(
+            pn.pane.Bokeh(
+                row(
+                    select_axis,
+                    select_cbar,
+                )
+            ),
+            pn.pane.Bokeh(
+                row(select_data_type)
+            ),
+            pn.pane.Bokeh(
+                row(
+                    fig,
+                    slider_index,
+                )
+            )
+        )
+
+        display(app)
 
 
 def plot_2d_image(
@@ -1142,18 +1185,18 @@ def plot_2d_image(
     x_label="x",
     y_label="y"
 ):
-    """Plot 2d image from 2d array.
+    """
+    Plot 2d image from 2d array.
 
     :param two_d_array: np.ndarray to plot, must be 2D
     :param fontsize: default to 15
     :param fig: plt.figure to plot in, default is None and
-     will create a figure
+     will create a figure, used to add 2d images to 3d figures
     :param ax: axes of figure, default is None and will create axes
     :param log: True to have a logarithmic scale, False to have a linear scale
     :param cmap: matplotlib cmap used for plot, default 'YlGnBu_r'
     :param title: str, title for this axe
-    :return: image
-
+    :return: image on ax (ax.imshow())
     """
 
     if not fig and not ax:
@@ -1167,10 +1210,6 @@ def plot_2d_image(
             norm={"linear": None, "logarithmic": LogNorm()}[
                 scale],
             cmap=cmap,
-            # cmap="cividis",
-            # extent=(0, 2, 0, 2),
-            # vmin=dmin,
-            # vmax=dmax,
         )
         ax.set_xlabel(x_label, fontsize=fontsize)
         ax.set_ylabel(y_label, fontsize=fontsize)
@@ -1180,7 +1219,6 @@ def plot_2d_image(
         return img
 
     except TypeError:
-        # plt.close()
         print("Using complex data, automatically switching to array module")
 
         img = ax.imshow(
@@ -1188,10 +1226,6 @@ def plot_2d_image(
             norm={"linear": None, "logarithmic": LogNorm()}[
                 scale],
             cmap=cmap,
-            # cmap="cividis",
-            # extent=(0, 2, 0, 2),
-            # vmin=dmin,
-            # vmax=dmax,
         )
         ax.set_xlabel(x_label, fontsize=fontsize)
         ax.set_ylabel(y_label, fontsize=fontsize)
@@ -1217,12 +1251,13 @@ def plot_3d_slices(
     cmap="YlGnBu_r",
     title=None
 ):
-    """Plot 3 slices for 3d data.
+    """
+    Plot 3 slices for 3d data.
 
-    :param data_array: np.ndarray to plotn must be 3D
+    :param data_array: np.ndarray to plot, must be 3D
     :param fontsize: default 15
     :param figsize: default (10, 10)
-    :param log: boolean (True, False) or anything else which
+    :param log: boolean (True, False) or, otherwise
      raises an interactive window
     :param cmap: matplotlib cmap used for plot, default 'YlGnBu_r'
     :param title: string to set as title for main plot or list of
@@ -1236,6 +1271,7 @@ def plot_3d_slices(
 
         fig, axs = plt.subplots(1, 3, figsize=figsize)
 
+        # Add titles
         if isinstance(title, str):
             fig.suptitle(title, fontsize=fontsize + 2)
             titles = [None, None, None]
@@ -1247,6 +1283,7 @@ def plot_3d_slices(
         # Each axis has a dimension
         shape = data_array.shape
 
+        # Plot first image
         two_d_array = data_array[shape[0]//2, :, :]
         img_x = plot_2d_image(two_d_array, fig=fig, title=titles[0],
                               ax=axs[0], log=log, cmap=cmap, fontsize=fontsize,
@@ -1259,6 +1296,7 @@ def plot_3d_slices(
         # Create colorbar
         fig.colorbar(mappable=img_x, cax=cbar_ax)
 
+        # Plot second image
         two_d_array = data_array[:, shape[1]//2, :]
         img_y = plot_2d_image(two_d_array, fig=fig, title=titles[1],
                               ax=axs[1], log=log, cmap=cmap, fontsize=fontsize,
@@ -1271,6 +1309,7 @@ def plot_3d_slices(
         # Create colorbar
         fig.colorbar(mappable=img_y, cax=cbar_ax)
 
+        # Plot third image
         two_d_array = data_array[:, :, shape[2]//2]
         img_z = plot_2d_image(two_d_array, fig=fig, title=titles[2],
                               ax=axs[2], log=log, cmap=cmap, fontsize=fontsize,
