@@ -16,7 +16,6 @@ from IPython.display import display, Markdown, Latex, clear_output, Image
 
 # gwaihir package
 import gwaihir
-from gwaihir.gui import gui_iterable
 
 # bcdi package
 from bcdi.preprocessing import ReadNxs3 as rd
@@ -463,8 +462,8 @@ def extract_metadata(
      bcdi.preprocessing_BCDI.py script
     :param gwaihir_dataset: Dataset object in which the metadata is saved,
      optionnal
-    :param metadata_csv_file: csv file in which the metadata is saved. If
-     None, defaulted to gwaihir_dataset.root_folder + "/metadata.csv"
+    :param metadata_csv_file: csv file in which the metadata is saved.
+     If None, defaulted to os.getcwd() + "/metadata.csv"
     """
 
     # Open file
@@ -608,6 +607,7 @@ def create_yaml_file(fname, **kwargs):
 
     file = os.path.basename(fname)
     directory = fname.strip(file)
+
     # Create directory
     if not os.path.isdir(directory):
         full_path = ""
@@ -618,6 +618,7 @@ def create_yaml_file(fname, **kwargs):
             except (FileExistsError, PermissionError):
                 pass
 
+    # Save in file
     with open(fname, "w") as v:
         for line in config_file:
             v.write(line + "\n")
